@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent-cell-renderer.c 10505 2010-04-21 04:38:54Z charles $
+ * $Id: torrent-cell-renderer.c 11747 2011-01-22 13:26:44Z jordan $
  */
 
 #include <string.h> /* strcmp() */
@@ -44,12 +44,12 @@ enum
 static char*
 getProgressString( const tr_torrent * tor,
                    const tr_info    * info,
-                   const tr_stat    * torStat )
+                   const tr_stat    * st )
 {
-    const int      isDone = torStat->leftUntilDone == 0;
-    const uint64_t haveTotal = torStat->haveUnchecked + torStat->haveValid;
-    const int      isSeed = torStat->haveValid >= info->totalSize;
-    char           buf1[32], buf2[32], buf3[32], buf4[32], buf5[32];
+    const int      isDone = st->leftUntilDone == 0;
+    const uint64_t haveTotal = st->haveUnchecked + st->haveValid;
+    const int      isSeed = st->haveValid >= info->totalSize;
+    char           buf1[32], buf2[32], buf3[32], buf4[32], buf5[32], buf6[32];
     char *         str;
     double         seedRatio;
     const gboolean hasSeedRatio = tr_torrentGetSeedRatio( tor, &seedRatio );
@@ -59,11 +59,11 @@ getProgressString( const tr_torrent * tor,
         str = g_strdup_printf(
             /* %1$s is how much we've got,
                %2$s is how much we'll have when done,
-               %3$.2f%% is a percentage of the two */
-            _( "%1$s of %2$s (%3$.2f%%)" ),
+               %3$s%% is a percentage of the two */
+            _( "%1$s of %2$s (%3$s%%)" ),
             tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
-            tr_strlsize( buf2, torStat->sizeWhenDone, sizeof( buf2 ) ),
-            tr_truncd( torStat->percentDone * 100.0, 2 ) );
+            tr_strlsize( buf2, st->sizeWhenDone, sizeof( buf2 ) ),
+            tr_strlpercent( buf3, st->percentDone * 100.0, sizeof( buf3 ) ) );
     }
     else if( !isSeed ) /* partial seeds */
     {
@@ -72,32 +72,32 @@ getProgressString( const tr_torrent * tor,
             str = g_strdup_printf(
                 /* %1$s is how much we've got,
                    %2$s is the torrent's total size,
-                   %3$.2f%% is a percentage of the two,
+                   %3$s%% is a percentage of the two,
                    %4$s is how much we've uploaded,
                    %5$s is our upload-to-download ratio,
                    %6$s is the ratio we want to reach before we stop uploading */
-                _( "%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s Goal: %6$s)" ),
+                _( "%1$s of %2$s (%3$s%%), uploaded %4$s (Ratio: %5$s Goal: %6$s)" ),
                 tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
                 tr_strlsize( buf2, info->totalSize, sizeof( buf2 ) ),
-                tr_truncd( torStat->percentComplete * 100.0, 2 ),
-                tr_strlsize( buf3, torStat->uploadedEver, sizeof( buf3 ) ),
-                tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ),
-                tr_strlratio( buf5, seedRatio, sizeof( buf5 ) ) );
+                tr_strlpercent( buf3, st->percentComplete * 100.0, sizeof( buf3 ) ),
+                tr_strlsize( buf4, st->uploadedEver, sizeof( buf4 ) ),
+                tr_strlratio( buf5, st->ratio, sizeof( buf5 ) ),
+                tr_strlratio( buf6, seedRatio, sizeof( buf6 ) ) );
         }
         else
         {
             str = g_strdup_printf(
                 /* %1$s is how much we've got,
                    %2$s is the torrent's total size,
-                   %3$.2f%% is a percentage of the two,
+                   %3$s%% is a percentage of the two,
                    %4$s is how much we've uploaded,
                    %5$s is our upload-to-download ratio */
-                _( "%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s)" ),
+                _( "%1$s of %2$s (%3$s%%), uploaded %4$s (Ratio: %5$s)" ),
                 tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
                 tr_strlsize( buf2, info->totalSize, sizeof( buf2 ) ),
-                tr_truncd( torStat->percentComplete * 100.0, 2 ),
-                tr_strlsize( buf3, torStat->uploadedEver, sizeof( buf3 ) ),
-                tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ) );
+                tr_strlpercent( buf3, st->percentComplete * 100.0, sizeof( buf3 ) ),
+                tr_strlsize( buf4, st->uploadedEver, sizeof( buf4 ) ),
+                tr_strlratio( buf5, st->ratio, sizeof( buf5 ) ) );
         }
     }
     else /* seeding */
@@ -111,8 +111,8 @@ getProgressString( const tr_torrent * tor,
                    %4$s is the ratio we want to reach before we stop uploading */
                 _( "%1$s, uploaded %2$s (Ratio: %3$s Goal: %4$s)" ),
                 tr_strlsize( buf1, info->totalSize, sizeof( buf1 ) ),
-                tr_strlsize( buf2, torStat->uploadedEver, sizeof( buf2 ) ),
-                tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ),
+                tr_strlsize( buf2, st->uploadedEver, sizeof( buf2 ) ),
+                tr_strlratio( buf3, st->ratio, sizeof( buf3 ) ),
                 tr_strlratio( buf4, seedRatio, sizeof( buf4 ) ) );
         }
         else /* seeding w/o a ratio */
@@ -123,16 +123,16 @@ getProgressString( const tr_torrent * tor,
                    %3$s is our upload-to-download ratio */
                 _( "%1$s, uploaded %2$s (Ratio: %3$s)" ),
                 tr_strlsize( buf1, info->totalSize, sizeof( buf1 ) ),
-                tr_strlsize( buf2, torStat->uploadedEver, sizeof( buf2 ) ),
-                tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ) );
+                tr_strlsize( buf2, st->uploadedEver, sizeof( buf2 ) ),
+                tr_strlratio( buf3, st->ratio, sizeof( buf3 ) ) );
         }
     }
 
     /* add time when downloading */
-    if( ( torStat->activity == TR_STATUS_DOWNLOAD )
-        || ( hasSeedRatio && ( torStat->activity == TR_STATUS_SEED ) ) )
+    if( ( st->activity == TR_STATUS_DOWNLOAD )
+        || ( hasSeedRatio && ( st->activity == TR_STATUS_SEED ) ) )
     {
-        const int eta = torStat->eta;
+        const int eta = st->eta;
         GString * gstr = g_string_new( str );
         g_string_append( gstr, " - " );
         if( eta < 0 )
@@ -153,24 +153,24 @@ getProgressString( const tr_torrent * tor,
 
 static char*
 getShortTransferString( const tr_torrent  * tor,
-                        const tr_stat     * torStat,
-                        double              uploadSpeed,
-                        double              downloadSpeed,
+                        const tr_stat     * st,
+                        double              uploadSpeed_KBps,
+                        double              downloadSpeed_KBps,
                         char              * buf,
                         size_t              buflen )
 {
     char downStr[32], upStr[32];
     const int haveMeta = tr_torrentHasMetadata( tor );
-    const int haveDown = haveMeta && torStat->peersSendingToUs > 0;
-    const int haveUp = haveMeta && torStat->peersGettingFromUs > 0;
+    const int haveUp = haveMeta && st->peersGettingFromUs > 0;
+    const int haveDown = haveMeta && ( ( st->peersSendingToUs > 0 ) || ( st->webseedsSendingToUs > 0 ) );
 
     if( haveDown )
-        tr_strlspeed( downStr, downloadSpeed, sizeof( downStr ) );
+        tr_formatter_speed_KBps( downStr, downloadSpeed_KBps, sizeof( downStr ) );
     if( haveUp )
-        tr_strlspeed( upStr, uploadSpeed, sizeof( upStr ) );
+        tr_formatter_speed_KBps( upStr, uploadSpeed_KBps, sizeof( upStr ) );
 
     if( haveDown && haveUp )
-        /* 1==down speed, 2==down arrow, 3==up speed, 4==down arrow */
+        /* 1==down arrow, 2==down speed, 3==up arrow, 4==down speed */
         g_snprintf( buf, buflen, _( "%1$s %2$s, %3$s %4$s" ),
                     gtr_get_unicode_string( GTR_UNICODE_DOWN ), downStr,
                     gtr_get_unicode_string( GTR_UNICODE_UP ), upStr );
@@ -182,7 +182,7 @@ getShortTransferString( const tr_torrent  * tor,
         /* bandwidth speed + unicode arrow */
         g_snprintf( buf, buflen, _( "%1$s %2$s" ),
                     gtr_get_unicode_string( GTR_UNICODE_UP ), upStr );
-    else if( tr_torrentHasMetadata( tor ) )
+    else if( haveMeta )
         /* the torrent isn't uploading or downloading */
         g_strlcpy( buf, _( "Idle" ), buflen );
     else
@@ -193,16 +193,16 @@ getShortTransferString( const tr_torrent  * tor,
 
 static char*
 getShortStatusString( const tr_torrent  * tor,
-                      const tr_stat     * torStat,
-                      double              uploadSpeed,
-                      double              downloadSpeed )
+                      const tr_stat     * st,
+                      double              uploadSpeed_KBps,
+                      double              downloadSpeed_KBps )
 {
     GString * gstr = g_string_new( NULL );
 
-    switch( torStat->activity )
+    switch( st->activity )
     {
         case TR_STATUS_STOPPED:
-            if( torStat->finished )
+            if( st->finished )
                 g_string_assign( gstr, _( "Finished" ) );
             else
                 g_string_assign( gstr, _( "Paused" ) );
@@ -215,20 +215,20 @@ getShortStatusString( const tr_torrent  * tor,
         case TR_STATUS_CHECK:
             g_string_append_printf( gstr,
                                     _( "Verifying local data (%.1f%% tested)" ),
-                                    tr_truncd( torStat->recheckProgress * 100.0, 1 ) );
+                                    tr_truncd( st->recheckProgress * 100.0, 1 ) );
             break;
 
         case TR_STATUS_DOWNLOAD:
         case TR_STATUS_SEED:
         {
             char buf[512];
-            if( torStat->activity != TR_STATUS_DOWNLOAD )
+            if( st->activity != TR_STATUS_DOWNLOAD )
             {
-                tr_strlratio( buf, torStat->ratio, sizeof( buf ) );
+                tr_strlratio( buf, st->ratio, sizeof( buf ) );
                 g_string_append_printf( gstr, _( "Ratio %s" ), buf );
                 g_string_append( gstr, ", " );
             }
-            getShortTransferString( tor, torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
+            getShortTransferString( tor, st, uploadSpeed_KBps, downloadSpeed_KBps, buf, sizeof( buf ) );
             g_string_append( gstr, buf );
             break;
         }
@@ -242,30 +242,30 @@ getShortStatusString( const tr_torrent  * tor,
 
 static char*
 getStatusString( const tr_torrent  * tor,
-                 const tr_stat     * torStat,
-                 const double        uploadSpeed,
-                 const double        downloadSpeed )
+                 const tr_stat     * st,
+                 const double        uploadSpeed_KBps,
+                 const double        downloadSpeed_KBps )
 {
-    const int isActive = torStat->activity != TR_STATUS_STOPPED;
-    const int isChecking = torStat->activity == TR_STATUS_CHECK
-                        || torStat->activity == TR_STATUS_CHECK_WAIT;
+    const int isActive = st->activity != TR_STATUS_STOPPED;
+    const int isChecking = st->activity == TR_STATUS_CHECK
+                        || st->activity == TR_STATUS_CHECK_WAIT;
 
     GString * gstr = g_string_new( NULL );
 
-    if( torStat->error )
+    if( st->error )
     {
         const char * fmt[] = { NULL, N_( "Tracker gave a warning: \"%s\"" ),
                                      N_( "Tracker gave an error: \"%s\"" ),
                                      N_( "Error: %s" ) };
-        g_string_append_printf( gstr, _( fmt[torStat->error] ), torStat->errorString );
+        g_string_append_printf( gstr, _( fmt[st->error] ), st->errorString );
     }
-    else switch( torStat->activity )
+    else switch( st->activity )
     {
         case TR_STATUS_STOPPED:
         case TR_STATUS_CHECK_WAIT:
         case TR_STATUS_CHECK:
         {
-            char * pch = getShortStatusString( tor, torStat, uploadSpeed, downloadSpeed );
+            char * pch = getShortStatusString( tor, st, uploadSpeed_KBps, downloadSpeed_KBps );
             g_string_assign( gstr, pch );
             g_free( pch );
             break;
@@ -276,40 +276,38 @@ getStatusString( const tr_torrent  * tor,
             if( tr_torrentHasMetadata( tor ) )
             {
                 g_string_append_printf( gstr,
-                    ngettext( "Downloading from %1$'d of %2$'d connected peer",
-                              "Downloading from %1$'d of %2$'d connected peers",
-                              torStat->peersConnected ),
-                    torStat->peersSendingToUs +
-                    torStat->webseedsSendingToUs,
-                    torStat->peersConnected +
-                    torStat->webseedsSendingToUs );
+                    gtr_ngettext( "Downloading from %1$'d of %2$'d connected peer",
+                                  "Downloading from %1$'d of %2$'d connected peers",
+                                  st->webseedsSendingToUs + st->peersSendingToUs ),
+                    st->webseedsSendingToUs + st->peersSendingToUs,
+                    st->webseedsSendingToUs + st->peersConnected );
             }
             else
             {
                 g_string_append_printf( gstr,
-                    ngettext( "Downloading metadata from %1$'d peer (%2$d%% done)",
-                              "Downloading metadata from %1$'d peers (%2$d%% done)",
-                              torStat->peersConnected ),
-                    torStat->peersConnected + torStat->webseedsSendingToUs,
-                    (int)(100.0*torStat->metadataPercentComplete) );
+                    gtr_ngettext( "Downloading metadata from %1$'d peer (%2$d%% done)",
+                                  "Downloading metadata from %1$'d peers (%2$d%% done)",
+                                  st->peersConnected + st->peersConnected ),
+                    st->peersConnected + st->webseedsSendingToUs,
+                    (int)(100.0*st->metadataPercentComplete) );
             }
             break;
         }
 
         case TR_STATUS_SEED:
             g_string_append_printf( gstr,
-                ngettext( "Seeding to %1$'d of %2$'d connected peer",
-                          "Seeding to %1$'d of %2$'d connected peers",
-                          torStat->peersConnected ),
-                torStat->peersGettingFromUs,
-                torStat->peersConnected );
+                gtr_ngettext( "Seeding to %1$'d of %2$'d connected peer",
+                              "Seeding to %1$'d of %2$'d connected peers",
+                              st->peersConnected ),
+                st->peersGettingFromUs,
+                st->peersConnected );
                 break;
     }
 
     if( isActive && !isChecking )
     {
         char buf[256];
-        getShortTransferString( tor, torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
+        getShortTransferString( tor, st, uploadSpeed_KBps, downloadSpeed_KBps, buf, sizeof( buf ) );
         if( *buf )
             g_string_append_printf( gstr, " - %s", buf );
     }
@@ -327,19 +325,18 @@ struct TorrentCellRendererPrivate
 {
     tr_torrent       * tor;
     GtkCellRenderer  * text_renderer;
-    GtkCellRenderer  * text_renderer_err;
     GtkCellRenderer  * progress_renderer;
     GtkCellRenderer  * icon_renderer;
     int bar_height;
 
-    /* use this instead of tr_stat.pieceUploadSpeed so that the model can
-       control when the speed displays get updated.  this is done to keep
+    /* Use this instead of tr_stat.pieceUploadSpeed so that the model can
+       control when the speed displays get updated. This is done to keep
        the individual torrents' speeds and the status bar's overall speed
        in sync even if they refresh at slightly different times */
-    double upload_speed;
+    double upload_speed_KBps;
 
-    /* @see upload_speed */
-    double download_speed;
+    /* @see upload_speed_Bps */
+    double download_speed_KBps;
 
     gboolean compact;
 };
@@ -361,15 +358,9 @@ get_icon( const tr_torrent * tor, GtkIconSize icon_size, GtkWidget * for_widget
     else if( strchr( info->files[0].name, '/' ) != NULL )
         mime_type = DIRECTORY_MIME_TYPE;
     else
-        mime_type = get_mime_type_from_filename( info->files[0].name );
+        mime_type = gtr_get_mime_type_from_filename( info->files[0].name );
 
-    return get_mime_type_icon( mime_type, icon_size, for_widget );
-}
-
-static GtkCellRenderer*
-get_text_renderer( const tr_stat * st, TorrentCellRenderer * r )
-{
-    return st->error ? r->priv->text_renderer_err : r->priv->text_renderer;
+    return gtr_get_mime_type_icon( mime_type, icon_size, for_widget );
 }
 
 /***
@@ -383,13 +374,13 @@ get_size_compact( TorrentCellRenderer * cell,
                   gint                * height )
 {
     int w, h;
+    int xpad, ypad;
     GdkRectangle icon_area;
     GdkRectangle name_area;
     GdkRectangle stat_area;
     const char * name;
     char * status;
     GdkPixbuf * icon;
-    GtkCellRenderer * text_renderer;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
@@ -397,20 +388,20 @@ get_size_compact( TorrentCellRenderer * cell,
 
     icon = get_icon( tor, COMPACT_ICON_SIZE, widget );
     name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( tor, st, p->upload_speed, p->download_speed );
+    status = getShortStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
+    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
     gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
     icon_area.width = w;
     icon_area.height = h;
-    text_renderer = get_text_renderer( st, cell );
-    g_object_set( text_renderer, "text", name, "ellipsize", PANGO_ELLIPSIZE_NONE,  "scale", 1.0, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", name, "ellipsize", PANGO_ELLIPSIZE_NONE,  "scale", 1.0, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     name_area.width = w;
     name_area.height = h;
-    g_object_set( text_renderer, "text", status, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", status, "scale", SMALL_SCALE, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
 
@@ -420,9 +411,9 @@ get_size_compact( TorrentCellRenderer * cell,
 
 #define BAR_WIDTH 50
     if( width != NULL )
-        *width = cell->parent.xpad * 2 + icon_area.width + GUI_PAD + name_area.width + GUI_PAD + BAR_WIDTH + GUI_PAD + stat_area.width;
+        *width = xpad * 2 + icon_area.width + GUI_PAD + name_area.width + GUI_PAD + BAR_WIDTH + GUI_PAD + stat_area.width;
     if( height != NULL )
-        *height = cell->parent.ypad * 2 + MAX( name_area.height, p->bar_height );
+        *height = ypad * 2 + MAX( name_area.height, p->bar_height );
 
     /* cleanup */
     g_free( status );
@@ -438,6 +429,7 @@ get_size_full( TorrentCellRenderer * cell,
                gint                * height )
 {
     int w, h;
+    int xpad, ypad;
     GdkRectangle icon_area;
     GdkRectangle name_area;
     GdkRectangle stat_area;
@@ -446,7 +438,6 @@ get_size_full( TorrentCellRenderer * cell,
     char * status;
     char * progress;
     GdkPixbuf * icon;
-    GtkCellRenderer * text_renderer;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
@@ -455,25 +446,25 @@ get_size_full( TorrentCellRenderer * cell,
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
     name = inf->name;
-    status = getStatusString( tor, st, p->upload_speed, p->download_speed );
+    status = getStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
     progress = getProgressString( tor, inf, st );
+    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
     gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
     icon_area.width = w;
     icon_area.height = h;
-    text_renderer = get_text_renderer( st, cell );
-    g_object_set( text_renderer, "text", name, "weight", PANGO_WEIGHT_BOLD, "scale", 1.0, "ellipsize", PANGO_ELLIPSIZE_NONE, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", name, "weight", PANGO_WEIGHT_BOLD, "scale", 1.0, "ellipsize", PANGO_ELLIPSIZE_NONE, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     name_area.width = w;
     name_area.height = h;
-    g_object_set( text_renderer, "text", progress, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", progress, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     prog_area.width = w;
     prog_area.height = h;
-    g_object_set( text_renderer, "text", status, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", status, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
 
@@ -482,9 +473,9 @@ get_size_full( TorrentCellRenderer * cell,
     **/
 
     if( width != NULL )
-        *width = cell->parent.xpad * 2 + icon_area.width + GUI_PAD + MAX3( name_area.width, prog_area.width, stat_area.width );
+        *width = xpad * 2 + icon_area.width + GUI_PAD + MAX3( name_area.width, prog_area.width, stat_area.width );
     if( height != NULL )
-        *height = cell->parent.ypad * 2 + name_area.height + prog_area.height + GUI_PAD_SMALL + p->bar_height + GUI_PAD_SMALL + stat_area.height;
+        *height = ypad * 2 + name_area.height + prog_area.height + GUI_PAD_SMALL + p->bar_height + GUI_PAD_SMALL + stat_area.height;
 
     /* cleanup */
     g_free( status );
@@ -506,8 +497,8 @@ torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
 
     if( self && self->priv->tor )
     {
-        struct TorrentCellRendererPrivate * p = self->priv;
         int w, h;
+        struct TorrentCellRendererPrivate * p = self->priv;
 
         if( p->compact )
             get_size_compact( TORRENT_CELL_RENDERER( cell ), widget, &w, &h );
@@ -520,16 +511,30 @@ torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
         if( height )
             *height = h;
 
-        if( cell_area ) {
-            if( x_offset ) *x_offset = 0;
-            if( y_offset ) {
-                *y_offset = 0.5 * ( cell_area->height - ( h + ( 2 * cell->ypad ) ) );
-                *y_offset = MAX( *y_offset, 0 );
-            }
+        if( x_offset )
+            *x_offset = cell_area ? cell_area->x : 0;
+
+        if( y_offset ) {
+            int xpad, ypad;
+            gtr_cell_renderer_get_padding( cell, &xpad, &ypad );
+            *y_offset = cell_area ? (int)((cell_area->height - (ypad*2 +h)) / 2.0) : 0;
         }
     }
 }
 
+static void
+get_text_color( GtkWidget * w, const tr_stat * st, GdkColor * setme )
+{
+    static const GdkColor red = { 0, 65535, 0, 0 };
+
+    if( st->error )
+        *setme = red;
+    else if( st->activity == TR_STATUS_STOPPED )
+        *setme = gtk_widget_get_style(w)->text[GTK_STATE_INSENSITIVE];
+    else
+        *setme = gtk_widget_get_style(w)->text[GTK_STATE_NORMAL];
+}
+
 static void
 render_compact( TorrentCellRenderer   * cell,
                 GdkDrawable           * window,
@@ -539,7 +544,7 @@ render_compact( TorrentCellRenderer   * cell,
                 GdkRectangle          * expose_area UNUSED,
                 GtkCellRendererState    flags )
 {
-    int w, h;
+    int xpad, ypad;
     GdkRectangle icon_area;
     GdkRectangle name_area;
     GdkRectangle stat_area;
@@ -548,7 +553,7 @@ render_compact( TorrentCellRenderer   * cell,
     const char * name;
     char * status;
     GdkPixbuf * icon;
-    GtkCellRenderer * text_renderer;
+    GdkColor text_color;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
@@ -559,58 +564,30 @@ render_compact( TorrentCellRenderer   * cell,
 
     icon = get_icon( tor, COMPACT_ICON_SIZE, widget );
     name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( tor, st, p->upload_speed, p->download_speed );
-
-    /* get the cell dimensions */
-    g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
-    gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
-    icon_area.width = w;
-    icon_area.height = h;
-    text_renderer = get_text_renderer( st, cell );
-    g_object_set( text_renderer, "text", name, "ellipsize", PANGO_ELLIPSIZE_NONE, "scale", 1.0, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    name_area.width = w;
-    name_area.height = h;
-    g_object_set( text_renderer, "text", status, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    stat_area.width = w;
-    stat_area.height = h;
-    prog_area.height = p->bar_height;
-
-    h = 1;
-    h = MAX( h, stat_area.height );
-    h = MAX( h, name_area.height );
-    h = MAX( h, icon_area.height );
-    h = MAX( h, prog_area.height );
-
-    /**
-    *** LAYOUT
-    **/
+    status = getShortStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
+    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
+    get_text_color( widget, st, &text_color );
 
     fill_area = *background_area;
-    fill_area.x += cell->parent.xpad;
-    fill_area.y += cell->parent.ypad;
-    fill_area.width -= cell->parent.xpad * 2;
-    fill_area.height -= cell->parent.ypad * 2;
+    fill_area.x += xpad;
+    fill_area.y += ypad;
+    fill_area.width -= xpad * 2;
+    fill_area.height -= ypad * 2;
+    icon_area = name_area = stat_area = prog_area = fill_area;
+
+    g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
+    gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &icon_area.width, NULL );
+    g_object_set( p->text_renderer, "text", name, "ellipsize", PANGO_ELLIPSIZE_NONE, "scale", 1.0, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &name_area.width, NULL );
+    g_object_set( p->text_renderer, "text", status, "scale", SMALL_SCALE, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &stat_area.width, NULL );
 
-    /* icon */
     icon_area.x = fill_area.x;
-    icon_area.y = fill_area.y;
-    icon_area.height = h;
-
-    /* progressbar */
     prog_area.x = fill_area.x + fill_area.width - BAR_WIDTH;
-    prog_area.y = fill_area.y;
     prog_area.width = BAR_WIDTH;
-    prog_area.height = h;
-
-    /* short status (right justified) */
     stat_area.x = prog_area.x - GUI_PAD - stat_area.width;
-    stat_area.y = fill_area.y + ( h - stat_area.height ) / 2;
-
-    /* name */
     name_area.x = icon_area.x + icon_area.width + GUI_PAD;
-    name_area.y = fill_area.y + ( h - name_area.height ) / 2;
+    name_area.y = fill_area.y;
     name_area.width = stat_area.x - GUI_PAD - name_area.x;
 
     /**
@@ -621,10 +598,10 @@ render_compact( TorrentCellRenderer   * cell,
     gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
     g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", NULL, "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->progress_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
-    g_object_set( text_renderer, "text", status, "scale", SMALL_SCALE, "sensitive", sensitive, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
-    gtk_cell_renderer_render( text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
-    g_object_set( text_renderer, "text", name, "scale", 1.0, NULL );
-    gtk_cell_renderer_render( text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
+    g_object_set( p->text_renderer, "text", status, "scale", SMALL_SCALE, "ellipsize", PANGO_ELLIPSIZE_END, "foreground-gdk", &text_color, NULL );
+    gtk_cell_renderer_render( p->text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
+    g_object_set( p->text_renderer, "text", name, "scale", 1.0, NULL );
+    gtk_cell_renderer_render( p->text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
 
     /* cleanup */
     g_free( status );
@@ -641,6 +618,7 @@ render_full( TorrentCellRenderer   * cell,
              GtkCellRendererState    flags )
 {
     int w, h;
+    int xpad, ypad;
     GdkRectangle fill_area;
     GdkRectangle icon_area;
     GdkRectangle name_area;
@@ -651,7 +629,7 @@ render_full( TorrentCellRenderer   * cell,
     char * status;
     char * progress;
     GdkPixbuf * icon;
-    GtkCellRenderer * text_renderer;
+    GdkColor text_color;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
@@ -663,25 +641,26 @@ render_full( TorrentCellRenderer   * cell,
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
     name = inf->name;
-    status = getStatusString( tor, st, p->upload_speed, p->download_speed );
+    status = getStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
     progress = getProgressString( tor, inf, st );
+    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
+    get_text_color( widget, st, &text_color );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
     gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
     icon_area.width = w;
     icon_area.height = h;
-    text_renderer = get_text_renderer( st, cell );
-    g_object_set( text_renderer, "text", name, "weight", PANGO_WEIGHT_BOLD, "ellipsize", PANGO_ELLIPSIZE_NONE, "scale", 1.0, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", name, "weight", PANGO_WEIGHT_BOLD, "ellipsize", PANGO_ELLIPSIZE_NONE, "scale", 1.0, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     name_area.width = w;
     name_area.height = h;
-    g_object_set( text_renderer, "text", progress, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", progress, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     prog_area.width = w;
     prog_area.height = h;
-    g_object_set( text_renderer, "text", status, NULL );
-    gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
+    g_object_set( p->text_renderer, "text", status, NULL );
+    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
 
@@ -690,10 +669,10 @@ render_full( TorrentCellRenderer   * cell,
     **/
 
     fill_area = *background_area;
-    fill_area.x += cell->parent.xpad;
-    fill_area.y += cell->parent.ypad;
-    fill_area.width -= cell->parent.xpad * 2;
-    fill_area.height -= cell->parent.ypad * 2;
+    fill_area.x += xpad;
+    fill_area.y += ypad;
+    fill_area.width -= xpad * 2;
+    fill_area.height -= ypad * 2;
 
     /* icon */
     icon_area.x = fill_area.x;
@@ -726,14 +705,14 @@ render_full( TorrentCellRenderer   * cell,
 
     g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
-    g_object_set( text_renderer, "text", name, "scale", 1.0, "sensitive", sensitive, "ellipsize", PANGO_ELLIPSIZE_END, "weight", PANGO_WEIGHT_BOLD, NULL );
-    gtk_cell_renderer_render( text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
-    g_object_set( text_renderer, "text", progress, "scale", SMALL_SCALE, "weight", PANGO_WEIGHT_NORMAL, NULL );
-    gtk_cell_renderer_render( text_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
+    g_object_set( p->text_renderer, "text", name, "scale", 1.0, "foreground-gdk", &text_color, "ellipsize", PANGO_ELLIPSIZE_END, "weight", PANGO_WEIGHT_BOLD, NULL );
+    gtk_cell_renderer_render( p->text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
+    g_object_set( p->text_renderer, "text", progress, "scale", SMALL_SCALE, "weight", PANGO_WEIGHT_NORMAL, NULL );
+    gtk_cell_renderer_render( p->text_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
     g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->progress_renderer, window, widget, &prct_area, &prct_area, &prct_area, flags );
-    g_object_set( text_renderer, "text", status, NULL );
-    gtk_cell_renderer_render( text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
+    g_object_set( p->text_renderer, "text", status, NULL );
+    gtk_cell_renderer_render( p->text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
 
     /* cleanup */
     g_free( status );
@@ -782,11 +761,11 @@ torrent_cell_renderer_set_property( GObject      * object,
 
     switch( property_id )
     {
-        case P_TORRENT:        p->tor            = g_value_get_pointer( v ); break;
-        case P_UPLOAD_SPEED:   p->upload_speed   = g_value_get_double( v ); break;
-        case P_DOWNLOAD_SPEED: p->download_speed = g_value_get_double( v ); break;
-        case P_BAR_HEIGHT:     p->bar_height     = g_value_get_int( v ); break;
-        case P_COMPACT:        p->compact        = g_value_get_boolean( v ); break;
+        case P_TORRENT:        p->tor                 = g_value_get_pointer( v ); break;
+        case P_UPLOAD_SPEED:   p->upload_speed_KBps   = g_value_get_double( v ); break;
+        case P_DOWNLOAD_SPEED: p->download_speed_KBps = g_value_get_double( v ); break;
+        case P_BAR_HEIGHT:     p->bar_height          = g_value_get_int( v ); break;
+        case P_COMPACT:        p->compact             = g_value_get_boolean( v ); break;
         default: G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec ); break;
     }
 }
@@ -803,8 +782,8 @@ torrent_cell_renderer_get_property( GObject     * object,
     switch( property_id )
     {
         case P_TORRENT:        g_value_set_pointer( v, p->tor ); break;
-        case P_UPLOAD_SPEED:   g_value_set_double( v, p->upload_speed ); break;
-        case P_DOWNLOAD_SPEED: g_value_set_double( v, p->download_speed ); break;
+        case P_UPLOAD_SPEED:   g_value_set_double( v, p->upload_speed_KBps ); break;
+        case P_DOWNLOAD_SPEED: g_value_set_double( v, p->download_speed_KBps ); break;
         case P_BAR_HEIGHT:     g_value_set_int( v, p->bar_height ); break;
         case P_COMPACT:        g_value_set_boolean( v, p->compact ); break;
         default: G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec ); break;
@@ -820,7 +799,6 @@ torrent_cell_renderer_dispose( GObject * o )
     if( r && r->priv )
     {
         g_object_unref( G_OBJECT( r->priv->text_renderer ) );
-        g_object_unref( G_OBJECT( r->priv->text_renderer_err ) );
         g_object_unref( G_OBJECT( r->priv->progress_renderer ) );
         g_object_unref( G_OBJECT( r->priv->icon_renderer ) );
         r->priv = NULL;
@@ -854,13 +832,13 @@ torrent_cell_renderer_class_init( TorrentCellRendererClass * klass )
 
     g_object_class_install_property( gobject_class, P_UPLOAD_SPEED,
                                     g_param_spec_double( "piece-upload-speed", NULL,
-                                                         "tr_stat.pieceUploadSpeed",
+                                                         "tr_stat.pieceUploadSpeed_KBps",
                                                          0, INT_MAX, 0,
                                                          G_PARAM_READWRITE ) );
 
     g_object_class_install_property( gobject_class, P_DOWNLOAD_SPEED,
                                     g_param_spec_double( "piece-download-speed", NULL,
-                                                         "tr_stat.pieceDownloadSpeed",
+                                                         "tr_stat.pieceDownloadSpeed_KBps",
                                                          0, INT_MAX, 0,
                                                          G_PARAM_READWRITE ) );
 
@@ -895,13 +873,9 @@ torrent_cell_renderer_init( GTypeInstance *  instance,
     p->tor = NULL;
     p->text_renderer = gtk_cell_renderer_text_new( );
     g_object_set( p->text_renderer, "xpad", 0, "ypad", 0, NULL );
-    p->text_renderer_err = gtk_cell_renderer_text_new(  );
-    g_object_set( p->text_renderer_err, "xpad", 0, "ypad", 0, NULL );
     p->progress_renderer = gtk_cell_renderer_progress_new(  );
     p->icon_renderer = gtk_cell_renderer_pixbuf_new(  );
-    g_object_set( p->text_renderer_err, "foreground", "red", NULL );
     gtr_object_ref_sink( p->text_renderer );
-    gtr_object_ref_sink( p->text_renderer_err );
     gtr_object_ref_sink( p->progress_renderer );
     gtr_object_ref_sink( p->icon_renderer );
 
